#include "MainWindow.h"
#include "ui_MainWindow.h"

#include <iostream>

using namespace std;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    connected(false),
    xValues(2048),
    yValues(2048),
    akfX(1024),
    akfY(1024)
{
    ui->setupUi(this);
    screenData = new char[2048];
    updateXValues();
    ui->graphTime->setAutoRefresh(true);
    ui->graphTime->setTitle(tr("Recorded sequence"), QFont("Helvetica", 10));
    ui->graphTime->setXLabel(tr("Time / s"), QFont("Helvetica", 10));
    ui->graphTime->setGrid(true);
    ui->graphTime->limitY(0, 255);

    ui->graphAC->setAutoRefresh(true);
    ui->graphAC->setTitle(tr("Autocorrelation function with highlighted maximums"), QFont("Helvetica", 10));
    ui->graphAC->setGrid(true);

    timer.setInterval(200);
    connect(&timer, SIGNAL(timeout()), this, SLOT(onButtonRefresh()));
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::onButtonOpen()
{
    if(ui->buttonOpen->text() == QString("Open"))
    {
        int error;
        if((error=hameg.openCom(ui->editPort->text(), ui->editBaud->text().toInt())) != 0)
        {
            QMessageBox::information(this, "ERROR", "Failed to open COM port: "+QString::number(error));
            return;
        }
        if(!hameg.checkConnection())
        {
            QMessageBox::information(this, "Warning: Interface not responding", "This interface is not responding!");
        }
        connected = true;
        ui->buttonOpen->setText(tr("Close"));
    }
    else
    {
        hameg.closeCom();
        connected = false;
        ui->buttonOpen->setText(tr("Open"));
    }
}

void MainWindow::onButtonRefresh()
{
    if(!connected)
        return;
    char channel;
    if(ui->comboChannel->currentIndex() == 0)
        channel = '1';
    else
        channel = '2';
    if(!hameg.getSingleShot(screenData, channel))
    {
        QMessageBox::information(this, "Error getting the screenshot of channel 1", "Failed to get the screenshot of channel 1");
    }
    updateXValues();
    for(int i=0; i<2048; i++)
        yValues[i] = (double)((unsigned char)screenData[i]);
    ui->graphTime->setData(xValues, yValues);
    ui->graphTime->refresh();

    ui->graphAC->clearData();
    Akf();
    findMaxs();
    ui->graphAC->appendData(akfX, akfY);
    ui->graphAC->refresh();
}

void MainWindow::onCheckAuto(bool autoRefresh)
{
    if(autoRefresh)
        timer.start();
    else
        timer.stop();
}

void MainWindow::updateXValues()
{
    double dx;
    switch(ui->comboTime->currentIndex())
    {
    case  0: dx = 5.00000; break;
    case  1: dx = 2.00000; break;
    case  2: dx = 1.00000; break;
    case  3: dx = 0.50000; break;
    case  4: dx = 0.20000; break;
    case  5: dx = 0.10000; break;
    case  6: dx = 0.05000; break;
    case  7: dx = 0.02000; break;
    case  8: dx = 0.01000; break;
    case  9: dx = 0.00500; break;
    case 10: dx = 0.00200; break;
    case 11: dx = 0.00100; break;
    case 12: dx = 0.00050; break;
    case 13: dx = 0.00020; break;
    case 14: dx = 0.00010; break;
    case 15: dx = 0.00005; break;
    case 16: dx = 0.00002; break;
    case 17: dx = 0.00001; break;
    default: dx = 1; break;
    }
    double factor = ui->editFactor->text().toDouble();
    if(factor<0.5 || factor>2)
        factor = 1.0;
    dx = dx/2048.0*10/factor;

    for(int i=0; i<2048; i++)
        xValues[i] = dx*i;

    for(int i=0; i<1024; i++)
        akfX[i] = dx*i;
}

void MainWindow::Akf()
{
    double min = std::numeric_limits<double>::max();
    for(int i=0; i<1024; i++)   // go through all samples
    {
        double sum = 0.0;
        for(int j=0; j<1024; j++)  // move of the current sample
        {
            sum += yValues[i+j]*yValues[j];
        }
        akfY[i] = sum;
        if(sum<min)
            min = sum;
    }
    for(int i=0; i<1024; i++)
        akfY[i] = (akfY[i]-min)/1024;
}

void MainWindow::findMaxs()
{
    // Fist find all local maximums
    QVector<int> localMaxs;
    //cout<<"Maxs: ";
    double globalMax = 0.0;
    for(int i=1; i<1023; i++)
    {
        if(akfY[i-1]<akfY[i] && akfY[i+1]<akfY[i])
        {
            localMaxs.push_back(i);
            //cout<<akfX[i]<<", ";
            if(akfY[i]>globalMax)
                globalMax = akfY[i];
        }
    }
    //cout<<endl;

    cout<<"Global max: "<<globalMax<<endl;
    // Remove all local max which are smaller than 0.9*globalMax
    QVector<int> localMaxsCut;
    for(int i=0; i<localMaxs.size(); i++)
    {
        if(akfY[localMaxs[i]] > 0.9*globalMax)
            localMaxsCut.push_back(localMaxs[i]);
    }

    // Plot max values
    QVector<double> x;
    QVector<double> y;
    for(int i=0; i<localMaxsCut.size(); i++)
    {
        x.push_back(akfX[localMaxsCut[i]]);
        y.push_back(akfY[localMaxsCut[i]]);
    }

    if(localMaxsCut.size()<2)
    {
        ui->labelFrequency->setText("??? Hz");
        return;
    }

    ui->graphAC->appendData(x, y, QGraph::Stem, 0.9, QPen(Qt::blue));
    ui->graphAC->refresh();

    // We guess that the largest distance between two maximums is the periode
    /*int distance = 0;
    for(int i=1; i<localMaxsCut.size(); i++)
    {
        if(localMaxsCut[i]-localMaxsCut[i-1] > distance)
            distance = localMaxsCut[i]-localMaxsCut[i-1];
    }*/
    double distance = localMaxsCut[localMaxsCut.size()-1] - localMaxsCut[0];
    distance /= localMaxsCut.size();
    double periode = akfX[distance];
    //cout<<"Per: "<<periode<<"s, Frequency: "<<1/periode<<"Hz"<<endl;
    ui->labelFrequency->setText(QString::number(1/periode)+" Hz");

}
